unit frmmain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ActnList,
  EditBtn, fppdfindexer, Inifiles;

type

  { TMainForm }

  TMainForm = class(TForm)
    actTestConn: TAction;
    actCreateTables: TAction;
    actIndex: TAction;
    alIndex: TActionList;
    btnCreate: TButton;
    btnIndex: TButton;
    btnTest: TButton;
    DEPDFs: TDirectoryEdit;
    edtLanguage: TEdit;
    edtHostname: TEdit;
    edtDatabaseName: TEdit;
    edtUsername: TEdit;
    edtPassword: TEdit;
    FEPDF: TFileNameEdit;
    FEIgnoreWords: TFileNameEdit;
    Label1: TLabel;
    lblIgnoreWords: TLabel;
    lblLog: TLabel;
    lblHostname: TLabel;
    lblDatabase: TLabel;
    lblUsername: TLabel;
    lblPassword: TLabel;
    mLog: TMemo;
    RBDir: TRadioButton;
    RBFile: TRadioButton;
    procedure actCreateTablesExecute(Sender: TObject);
    procedure actCreateTablesUpdate(Sender: TObject);
    procedure actIndexExecute(Sender: TObject);
    procedure actIndexUpdate(Sender: TObject);
    procedure actTestConnExecute(Sender: TObject);
    procedure actTestConnUpdate(Sender: TObject);
    procedure DoSelectDir(Sender: TObject);
    procedure DoSelectFile(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FIndexer : TPDFIndexer;
    FCanConnect : Boolean;
    FLastError : String;
    FFileCount : Integer;
    procedure DoIndexLog(Sender: TObject; const Msg: String);
    procedure DoLog(const Msg: string);
  public
    Function HaveConnectionData : Boolean;
    Procedure ConfigConnection;
    procedure ConfigDB(const aFileName: string);
    procedure IndexDirectory(const aDir: string);
    procedure IndexFile(const aFile: string);
    procedure SaveDBConfig(const aFileName: string);
    procedure TestConnection;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.lfm}

Const
  SIniFile = 'pdfindex.cfg';
  SSection = 'Database';
  KeyHost = 'Host';
  KeyDatabaseName = 'DatabaseName';
  KeyUserName = 'UserName';
  KeyPassword = 'Password';

{ TMainForm }

procedure TMainForm.DoIndexLog(Sender: TObject; const Msg: String);
begin
  DoLog(Msg);
end;

procedure TMainForm.DoLog(const Msg :  string);

begin
  mlog.Lines.Add(Msg);
end;

procedure TMainForm.actTestConnUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=HaveConnectionData;
end;

procedure TMainForm.DoSelectDir(Sender: TObject);
begin
  RBDir.Checked:=True;
end;

procedure TMainForm.DoSelectFile(Sender: TObject);
begin
  RBFile.Checked:=True;
end;

procedure TMainForm.actTestConnExecute(Sender: TObject);
begin
  TestConnection;
end;

procedure TMainForm.TestConnection;

begin
  ConfigConnection;
  Try
    FIndexer.Indexer.Connect;
    FIndexer.Indexer.Disconnect;
    FCanConnect:=True;
    SaveDBConfig(SIniFile);
  except
    on E : Exception do
      ShowMessage('Could not connect to index database:'#10+E.Message);
  end;
end;

procedure TMainForm.actCreateTablesUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=FCanConnect;
end;

procedure TMainForm.IndexFile(const aFile : string);

Const
  ErrMsg = 'Error %s while indexing %s: %s';

begin
  try
    FIndexer.Connect;
    try
      Inc(FFileCount);
      FIndexer.IndexPDF(aFile);
    finally
      Findexer.Disconnect;
    end;
  except
    On E : Exception do
      begin
      FLastError:=Format(ErrMsg,[E.ClassName,aFile,E.Message]);
      DoLog(FLastError);
      end;
  end;
  Application.ProcessMessages;
end;


procedure TMainForm.IndexDirectory(const aDir : string);

Var
  Info : TSearchRec;
  D : String;

begin
  D:=IncludeTrailingPathDelimiter(aDir);
  If FindFirst(D+'*.pdf',0,Info)=0 then
    try
      Repeat
        IndexFile(D+Info.Name);
      until FindNext(Info)<>0;
    finally
      FindClose(info)
    end;
end;


procedure TMainForm.actIndexExecute(Sender: TObject);

Var
  Msg : String;

begin
  if (FEIgnoreWords.FileName<>'') and FileExists(FEIgnoreWords.FileName) then
    FIndexer.IgnoreWords.LoadFromFile(FEIgnoreWords.FileName);
  FIndexer.Language:=edtLanguage.Text;
  FFileCount:=0;
  FLastError:='';
  if RBFile.Checked then
    IndexFile(FEPDF.FileName)
  else
    IndexDirectory(DEPDFs.Directory);
  Msg:=Format('Done indexing %d files.',[FFileCount]);
  if FlastError<>'' then
    Msg:=Msg+#10'There were errors:'#10+FLastError;
  ShowMessage(Msg)
end;

procedure TMainForm.actIndexUpdate(Sender: TObject);

Var
  HaveFile,HaveDir : Boolean;

begin
  HaveDir:=False;
  HaveFile:=RBFile.Checked and (FEPDF.FileName<>'');
  if not HaveFile then
    HaveDir:=RBDir.Checked and (DEPDFs.Directory<>'');
  (Sender as TAction).Enabled:=FCanConnect and (HaveDir or HaveFile);
end;

procedure TMainForm.actCreateTablesExecute(Sender: TObject);
begin
  ConfigConnection;
  FIndexer.CreateTables;
  FIndexer.Disconnect;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  FIndexer:=TPDFIndexer.Create(Self);
  FIndexer.OnLog:=@DoIndexLog;
  if FileExists(SIniFile) then
    ConfigDB(SIniFile)
end;

procedure TMainForm.ConfigDB(Const aFileName : string);

var
  aIni : TCustomIniFile;

begin
  aIni:=TMemIniFile.Create(aFileName);
  try
    With aIni do
      begin
      edtHostName.Text:=ReadString(SSection,KeyHost,edtHostName.Text);
      edtDatabaseName.Text:=ReadString(SSection,KeyDatabaseName,edtDatabaseName.Text);
      edtUserName.Text:=ReadString(SSection,KeyUserName,edtUserName.Text);
      edtPassword.Text:=ReadString(SSection,KeyPassword,edtPassword.Text);
      end;
  finally
    aIni.Free;
  end;
end;


procedure TMainForm.SaveDBConfig(Const aFileName : string);

var
  aIni : TCustomIniFile;

begin
  aIni:=TMemIniFile.Create(aFileName);
  try
    With aIni do
      begin
      WriteString(SSection,KeyHost,edtHostName.Text);
      WriteString(SSection,KeyDatabaseName,edtDatabaseName.Text);
      WriteString(SSection,KeyUserName,edtUserName.Text);
      WriteString(SSection,KeyPassword,edtPassword.Text);
      UpdateFile;
      end;
  finally
    aIni.Free;
  end;
end;


function TMainForm.HaveConnectionData: Boolean;
begin
  Result:=(edtHostname.Text<>'')
           and (edtDatabaseName.Text<>'')
           and (edtUserName.Text<>'')
           and (edtPassword.Text<>'');
end;

procedure TMainForm.ConfigConnection;
begin
   With Findexer.Indexer do
     begin
     DatabasePath:=edtDatabaseName.Text;
     Hostname:=edtHostname.Text;
     UserName:=edtUserName.Text;
     Password:=edtPassword.Text;
     end;
end;

end.

